@@ -336,7 +336,9 @@ handle_msg_start_emulating(struct ei_device *device, uint32_t serial, uint32_t s
 	case EI_DEVICE_STATE_EMULATING:
 	case EI_DEVICE_STATE_REMOVED_FROM_SERVER:
 		result = brei_result_new(EI_CONNECTION_DISCONNECT_REASON_PROTOCOL,
-					 "Invalid device state %u for a start_emulating event", device->state);
+					 "Invalid device state %u for a start_emulating event (sequence %u)",
+					 device->state,
+					 sequence);
 		break;
 	case EI_DEVICE_STATE_RESUMED:
 		ei_queue_device_start_emulating_event(device, sequence);
@@ -723,6 +725,7 @@ handle_msg_keymap(struct ei_keyboard *keyboard, uint32_t keymap_type, uint32_t k
 	struct ei_device *device = ei_keyboard_get_device(keyboard);
 
 	ei_device_set_keymap(device, keymap_type, keymap_fd, keymap_sz);
+	xclose (keymap_fd);
 
 	return NULL;
 }
