@@ -336,6 +336,19 @@ ei_queue_pong_event(struct ei *ei, struct ei_ping *ping)
 	queue_event(ei, e);
 }
 
+void
+ei_sync_event_send_done(struct ei_event *e)
+{
+	struct ei *ei = ei_event_get_context(e);
+	_unref_(ei_pingpong) *pp = steal(&e->sync.pingpong);
+	log_debug(ei_event_get_context(e),
+		  "object %#" PRIx64 ": ping pong done",
+		  ei_pingpong_get_id(pp));
+
+	if (ei->state < EI_STATE_DISCONNECTED)
+		ei_pingpong_request_done(pp, 0);
+}
+
 void
 ei_queue_sync_event(struct ei *ei, struct ei_pingpong *ping)
 {
@@ -568,7 +581,7 @@ ei_queue_touch_down_event(struct ei_device *device, uint32_t touchid,
 	struct ei_event *e = ei_event_new_for_device(device);
 
 	e->type = EI_EVENT_TOUCH_DOWN;
-	e->touch.touchid = touchid,
+	e->touch.touchid = touchid;
 	e->touch.x = x;
 	e->touch.y = y;
 
@@ -582,7 +595,7 @@ ei_queue_touch_motion_event(struct ei_device *device, uint32_t touchid,
 	struct ei_event *e = ei_event_new_for_device(device);
 
 	e->type = EI_EVENT_TOUCH_MOTION;
-	e->touch.touchid = touchid,
+	e->touch.touchid = touchid;
 	e->touch.x = x;
 	e->touch.y = y;
 
@@ -595,7 +608,7 @@ ei_queue_touch_up_event(struct ei_device *device, uint32_t touchid)
 	struct ei_event *e = ei_event_new_for_device(device);
 
 	e->type = EI_EVENT_TOUCH_UP;
-	e->touch.touchid = touchid,
+	e->touch.touchid = touchid;
 	e->touch.is_cancel = false;
 
 	queue_event(ei_device_get_context(device), e);
@@ -607,7 +620,7 @@ ei_queue_touch_cancel_event(struct ei_device *device, uint32_t touchid)
 	struct ei_event *e = ei_event_new_for_device(device);
 
 	e->type = EI_EVENT_TOUCH_UP;
-	e->touch.touchid = touchid,
+	e->touch.touchid = touchid;
 	e->touch.is_cancel = true;
 
 	queue_event(ei_device_get_context(device), e);
@@ -631,7 +644,7 @@ ei_disconnect(struct ei *ei)
 		ei_seat_remove(seat);
 	}
 
-	if (state != EI_STATE_NEW) {
+	if (state != EI_STATE_NEW && state != EI_STATE_BACKEND) {
 		ei_connection_request_disconnect(ei->connection);
 		ei_connection_remove_pending_callbacks(ei->connection);
 	}
@@ -693,16 +706,6 @@ ei_get_event(struct ei *ei)
 	struct ei_event *e = list_first_entry(&ei->event_queue, e, link);
 	list_remove(&e->link);
 
-	if (e->type == EI_EVENT_SYNC) {
-		_unref_(ei_pingpong) *pp = steal(&e->sync.pingpong);
-		log_debug(ei_event_get_context(e),
-			  "object %#" PRIx64 ": ping pong done",
-			  ei_pingpong_get_id(pp));
-
-		if (ei->state < EI_STATE_DISCONNECTED)
-			ei_pingpong_request_done(pp, 0);
-	}
-
 	return e;
 }
 
