@@ -727,9 +727,9 @@ ei_log_context_get_func(struct ei_log_context *ctx);
  * messages with a log level equal or greater than than the one set in
  * ei_log_set_priority().
  *
- * The context passed to this function contains auxilary information about
+ * The context passed to this function contains auxiliary information about
  * this log message such as the line number, file name and function name
- * this message occured in. The log context is valid only within the current
+ * this message occurred in. The log context is valid only within the current
  * invocation of the log handler.
  *
  * @param ei The EI context
@@ -901,8 +901,7 @@ ei_ping_ref(struct ei_ping *ei_ping);
 
 /**
  * Decrease the refcount of this struct by one. When the refcount reaches
- * zero, the context disconnects from the server and all allocated resources
- * are released.
+ * zero, all allocated resources for this struct are released.
  *
  * @return always NULL
  *
@@ -1046,7 +1045,7 @@ ei_seat_get_name(struct ei_seat *seat);
 /**
  * @ingroup libei-seat
  *
- * Return true if the capabilitiy is available on this seat or false
+ * Return true if the capability is available on this seat or false
  * otherwise. The return value of this function is not affected by
  * ei_seat_confirm_capability().
  */
@@ -1078,7 +1077,7 @@ __attribute__((sentinel));
 /**
  * @ingroup libei-seat
  *
- * Unbind a seat's capabilities, terminatd by ``NULL``.
+ * Unbind a seat's capabilities, terminated by ``NULL``.
  * This function indicates the the application is
  * no longer interested in devices with the given capability.
  *
@@ -1112,13 +1111,24 @@ struct ei *
 ei_seat_get_context(struct ei_seat *seat);
 
 /**
- * Release resources associated with this event. This function always
- * returns NULL.
+ * Increase the refcount of this struct by one. Use ei_event_unref() to decrease
+ * the refcount. This function always returns the same event that the reference
+ * was added to.
  *
- * The caller cannot increase the refcount of an event. Events should be
- * considered transient data and not be held longer than required.
- * ei_event_unref() is provided for consistency (as opposed to, say,
- * ei_event_free()).
+ * Events should be considered transient data and not be held longer than
+ * required.
+ */
+struct ei_event *
+ei_event_ref(struct ei_event *event);
+
+/**
+ * Decrease the refcount of this struct by one. When the refcount reaches
+ * zero, all allocated resources for this struct are released.
+ *
+ * Events should be considered transient data and not be held longer than
+ * required.
+ *
+ * @return always NULL
  */
 struct ei_event *
 ei_event_unref(struct ei_event *event);
@@ -1168,8 +1178,7 @@ ei_device_ref(struct ei_device *device);
  * @ingroup libei-device
  *
  * Decrease the refcount of this struct by one. When the refcount reaches
- * zero, the context disconnects from the server and all allocated resources
- * are released.
+ * zero, all allocated resources for this struct are released.
  *
  * @return always NULL
  */
@@ -1274,8 +1283,7 @@ ei_keymap_ref(struct ei_keymap *keymap);
  * @ingroup libei-keymap
  *
  * Decrease the refcount of this struct by one. When the refcount reaches
- * zero, the context disconnects from the server and all allocated resources
- * are released.
+ * zero, all allocated resources for this struct are released.
  *
  * @return always NULL
  */
@@ -1433,7 +1441,7 @@ ei_region_get_height(struct ei_region *region);
  * Get the unique identifier (representing an external resource) that is
  * attached to this region, if any. This is only available if the EIS
  * implementation supports version 2 or later of the ei_device protocol
- * interface *and* the EIS implementation chooses to attach such an identifer to
+ * interface *and* the EIS implementation chooses to attach such an identifier to
  * the region.
  *
  * This ID can be used by the client to identify an external resource that has a
@@ -1915,8 +1923,7 @@ ei_touch_ref(struct ei_touch *touch);
  * @ingroup libei-sender
  *
  * Decrease the refcount of this struct by one. When the refcount reaches
- * zero, the context disconnects from the server and all allocated resources
- * are released.
+ * zero, all allocated resources for this struct are released.
  *
  * @return always NULL
  */
