@@ -63,6 +63,7 @@ eis_event_destroy(struct eis_event *event)
 		handled = true;
 		break;
 	case EIS_EVENT_SYNC:
+		eis_sync_event_send_done(event);
 		eis_callback_unref(event->sync.callback);
 		handled = true;
 		break;
@@ -117,9 +118,8 @@ eis_event_new_for_device(struct eis_device *device)
 	return e;
 }
 
-/* this one is not public */
+_public_
 OBJECT_IMPLEMENT_REF(eis_event);
-
 _public_
 OBJECT_IMPLEMENT_UNREF_CLEANUP(eis_event);
 _public_
