@@ -464,7 +464,7 @@ portal_start_response_received(sd_bus_message *m, void *userdata, sd_bus_error *
 		return 0;
 	}
 
-	log_debug("Portal Start reponse is %u", response);
+	log_debug("Portal Start response is %u", response);
 	if (response != 0) {
 		oeffis_disconnect(oeffis, "Portal denied Start");
 		return 0;
@@ -541,7 +541,7 @@ portal_select_devices_response_received(sd_bus_message *m, void *userdata, sd_bu
 		return 0;
 	}
 
-	log_debug("Portal SelectDevices reponse is %u", response);
+	log_debug("Portal SelectDevices response is %u", response);
 	if (response != 0) {
 		oeffis_disconnect(oeffis, "Portal denied SelectDevices");
 		return 0;
@@ -614,7 +614,7 @@ portal_create_session_response_received(sd_bus_message *m, void *userdata, sd_bu
 		return 0;
 	}
 
-	log_debug("Portal CreateSession reponse is %u", response);
+	log_debug("Portal CreateSession response is %u", response);
 
 	const char *session_handle = NULL;
 	if (response == 0) {
