@@ -50,14 +50,22 @@ static bool enable_sigalarm;
 
 DEFINE_TRISTATE(yes, no, unset);
 
+struct peck_log_capture {
+	bool enabled;
+	char **debug;
+	char **info;
+	char **warning;
+	char **error;
+};
+
 struct peck {
 	struct object object;
 	struct ei *ei;
+	int ei_fd;
 	struct eis *eis;
 	uint32_t eis_behavior;
 	uint32_t ei_behavior;
 	struct logger *logger;
-	int ei_socket_fd;
 
 	/* The default seat/devices */
 	struct eis_seat *eis_seat;
@@ -89,6 +97,9 @@ struct peck {
 
 	uint64_t ei_time_offset;
 	uint64_t eis_time_offset;
+
+	struct peck_log_capture ei_log_capture;
+	struct peck_log_capture eis_log_capture;
 };
 
 static const uint32_t INDENTATION = 4;
@@ -182,6 +193,16 @@ peck_destroy(struct peck *peck)
 		setitimer(ITIMER_REAL, &timer, 0);
 		sigaction(SIGALRM, &peck->sigact, NULL);
 	}
+
+	strv_free(peck->ei_log_capture.debug);
+	strv_free(peck->ei_log_capture.info);
+	strv_free(peck->ei_log_capture.warning);
+	strv_free(peck->ei_log_capture.error);
+
+	strv_free(peck->eis_log_capture.debug);
+	strv_free(peck->eis_log_capture.info);
+	strv_free(peck->eis_log_capture.warning);
+	strv_free(peck->eis_log_capture.error);
 }
 
 void
@@ -203,6 +224,7 @@ OBJECT_IMPLEMENT_CREATE(peck);
 OBJECT_IMPLEMENT_UNREF(peck);
 OBJECT_IMPLEMENT_GETTER(peck, ei, struct ei*);
 OBJECT_IMPLEMENT_GETTER(peck, eis, struct eis*);
+OBJECT_IMPLEMENT_GETTER(peck, ei_fd, int);
 
 void
 peck_drop_ei(struct peck *peck)
@@ -215,6 +237,64 @@ peck_drop_ei(struct peck *peck)
 	peck->ei = NULL;
 }
 
+char **
+peck_ei_get_log_capture(struct peck *peck, enum ei_log_priority priority)
+{
+	switch (priority) {
+		case EI_LOG_PRIORITY_ERROR:
+			return peck->ei_log_capture.error;
+		case EI_LOG_PRIORITY_WARNING:
+			return peck->ei_log_capture.warning;
+		case EI_LOG_PRIORITY_INFO:
+			return peck->ei_log_capture.info;
+		case EI_LOG_PRIORITY_DEBUG:
+			return peck->ei_log_capture.debug;
+		default:
+			abort();
+	}
+}
+
+void
+peck_ei_enable_log_capture(struct peck *peck)
+{
+	peck->ei_log_capture.enabled = true;
+}
+
+void
+peck_ei_disable_log_capture(struct peck *peck)
+{
+	peck->ei_log_capture.enabled = true;
+}
+
+char **
+peck_eis_get_log_capture(struct peck *peck, enum eis_log_priority priority)
+{
+	switch (priority) {
+		case EIS_LOG_PRIORITY_ERROR:
+			return peck->eis_log_capture.error;
+		case EIS_LOG_PRIORITY_WARNING:
+			return peck->eis_log_capture.warning;
+		case EIS_LOG_PRIORITY_INFO:
+			return peck->eis_log_capture.info;
+		case EIS_LOG_PRIORITY_DEBUG:
+			return peck->eis_log_capture.debug;
+		default:
+			abort();
+	}
+}
+
+void
+peck_eis_enable_log_capture(struct peck *peck)
+{
+	peck->eis_log_capture.enabled = true;
+}
+
+void
+peck_eis_disable_log_capture(struct peck *peck)
+{
+	peck->eis_log_capture.enabled = true;
+}
+
 struct eis_client *
 peck_eis_get_default_client(struct peck *peck)
 {
@@ -361,6 +441,32 @@ peck_eis_enable_fatal_bug(struct peck *peck)
 	peck->eis_fatal_bugs = true;
 }
 
+static void
+peck_capture_log(struct peck_log_capture *capture,
+		 enum ei_log_priority priority,
+		 const char *message)
+{
+	if (!capture->enabled)
+		return;
+
+	switch (priority) {
+		case EI_LOG_PRIORITY_ERROR:
+			capture->error = strv_append_strdup(capture->error, message);
+			break;
+		case EI_LOG_PRIORITY_WARNING:
+			capture->info = strv_append_strdup(capture->info, message);
+			break;
+		case EI_LOG_PRIORITY_INFO:
+			capture->info = strv_append_strdup(capture->info, message);
+			break;
+		case EI_LOG_PRIORITY_DEBUG:
+			capture->debug = strv_append_strdup(capture->debug, message);
+			break;
+		default:
+			abort();
+	}
+}
+
 static void
 peck_ei_log_handler(struct ei *ei,
 		    enum ei_log_priority priority,
@@ -391,6 +497,8 @@ peck_ei_log_handler(struct ei *ei,
 		assert(!strstr(message, "🪳"));
 		assert(!strstr(message, "🪲"));
 	}
+
+	peck_capture_log(&peck->ei_log_capture, priority, message);
 }
 
 static void
@@ -423,6 +531,8 @@ peck_eis_log_handler(struct eis *eis,
 		assert(!strstr(message, "🪳"));
 		assert(!strstr(message, "🪲"));
 	}
+
+	peck_capture_log(&peck->eis_log_capture, (enum ei_log_priority)priority, message);
 }
 
 _printf_(7, 0)
@@ -467,10 +577,18 @@ handle_sigalrm(int signo)
 }
 
 static struct peck *
-new_context(enum peck_ei_mode ei_mode)
+new_context(const char *what, va_list args)
 {
 	struct peck *peck = peck_create(NULL);
 
+	enum peck_ei_mode ei_mode = PECK_EI_SENDER;
+
+	while (what) {
+		if (streq(what, "mode"))
+			ei_mode = va_arg(args, enum peck_ei_mode);
+		what = va_arg(args, const char *);
+	}
+
 	assert(ei_mode == PECK_EI_RECEIVER || ei_mode == PECK_EI_SENDER);
 
 	int rc, fd;
@@ -496,8 +614,8 @@ new_context(enum peck_ei_mode ei_mode)
 	ei_configure_name(ei, "eierpecken test context");
 	rc = ei_setup_backend_fd(ei, fd);
 	munit_assert_int(rc, ==, 0);
-	peck->ei_socket_fd = -1;
 	peck->ei = ei;
+	peck->ei_fd = fd;
 	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_CONNECT);
 	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTOSEAT);
 	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTOSTART);
@@ -545,26 +663,15 @@ new_context(enum peck_ei_mode ei_mode)
 }
 
 struct peck *
-peck_new_context(enum peck_ei_mode ei_mode)
+_peck_new_context(const char *what, ...)
 {
-	return new_context(ei_mode);
-}
+	va_list args;
 
-struct peck *
-peck_new(void)
-{
-	return peck_new_context(PECK_EI_SENDER);
-}
+	va_start(args, what);
+	struct peck *peck = new_context(what, args);
+	va_end(args);
 
-void
-peck_ei_connect(struct peck *peck)
-{
-	assert(peck->ei_socket_fd != -1);
-
-	struct ei *ei = peck->ei;
-	int rc = ei_setup_backend_fd(ei, peck->ei_socket_fd);
-	munit_assert_int(rc, ==, 0);
-	peck->ei_socket_fd = -1;
+	return peck;
 }
 
 void
@@ -593,10 +700,6 @@ peck_enable_eis_behavior(struct peck *peck, enum peck_eis_behavior behavior)
 		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ADD_KEYBOARD);
 		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ADD_TOUCH);
 		break;
-	case PECK_EIS_BEHAVIOR_REJECT_CLIENT:
-		flag_clear(peck->eis_behavior, behavior - 1);
-		flag_set(peck->eis_behavior, behavior);
-		break;
 	case PECK_EIS_BEHAVIOR_HANDLE_BIND_SEAT:
 	case PECK_EIS_BEHAVIOR_HANDLE_CLOSE_DEVICE:
 	case PECK_EIS_BEHAVIOR_HANDLE_FRAME:
@@ -609,8 +712,12 @@ peck_enable_eis_behavior(struct peck *peck, enum peck_eis_behavior behavior)
 	case PECK_EIS_BEHAVIOR_ADD_TOUCH:
 		flag_set(peck->eis_behavior, behavior);
 		break;
+	case PECK_EIS_BEHAVIOR_REJECT_CLIENT:
+		flag_clear(peck->eis_behavior, PECK_EIS_BEHAVIOR_ACCEPT_CLIENT);
+		flag_set(peck->eis_behavior, behavior);
+		break;
 	case PECK_EIS_BEHAVIOR_ACCEPT_CLIENT:
-		flag_clear(peck->eis_behavior, behavior + 1);
+		flag_clear(peck->eis_behavior, PECK_EIS_BEHAVIOR_REJECT_CLIENT);
 		flag_set(peck->eis_behavior, behavior);
 		break;
 	case PECK_EIS_BEHAVIOR_RESUME_DEVICE:
@@ -623,6 +730,52 @@ peck_enable_eis_behavior(struct peck *peck, enum peck_eis_behavior behavior)
 		break;
 	}
 }
+
+void
+peck_disable_eis_behavior(struct peck *peck, enum peck_eis_behavior behavior)
+{
+	switch (behavior) {
+	case PECK_EIS_BEHAVIOR_NONE:
+		peck->eis_behavior = 0;
+		break;
+	case PECK_EIS_BEHAVIOR_DEFAULT_SEAT:
+	case PECK_EIS_BEHAVIOR_NO_DEFAULT_SEAT:
+		flag_clear(peck->eis_behavior, behavior);
+		break;
+	case PECK_EIS_BEHAVIOR_ACCEPT_ALL:
+		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_DEFAULT_SEAT);
+		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ACCEPT_CLIENT);
+		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_HANDLE_BIND_SEAT);
+		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_RESUME_DEVICE);
+		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_HANDLE_CLOSE_DEVICE);
+		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_HANDLE_START_EMULATING);
+		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_HANDLE_STOP_EMULATING);
+		break;
+	case PECK_EIS_BEHAVIOR_ADD_DEVICES:
+		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ADD_POINTER);
+		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ADD_POINTER_ABSOLUTE);
+		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ADD_KEYBOARD);
+		peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ADD_TOUCH);
+		break;
+	case PECK_EIS_BEHAVIOR_HANDLE_BIND_SEAT:
+	case PECK_EIS_BEHAVIOR_HANDLE_CLOSE_DEVICE:
+	case PECK_EIS_BEHAVIOR_HANDLE_FRAME:
+	case PECK_EIS_BEHAVIOR_HANDLE_SYNC:
+	case PECK_EIS_BEHAVIOR_HANDLE_START_EMULATING:
+	case PECK_EIS_BEHAVIOR_HANDLE_STOP_EMULATING:
+	case PECK_EIS_BEHAVIOR_ADD_POINTER:
+	case PECK_EIS_BEHAVIOR_ADD_POINTER_ABSOLUTE:
+	case PECK_EIS_BEHAVIOR_ADD_KEYBOARD:
+	case PECK_EIS_BEHAVIOR_ADD_TOUCH:
+	case PECK_EIS_BEHAVIOR_REJECT_CLIENT:
+	case PECK_EIS_BEHAVIOR_ACCEPT_CLIENT:
+	case PECK_EIS_BEHAVIOR_RESUME_DEVICE:
+	case PECK_EIS_BEHAVIOR_SUSPEND_DEVICE:
+		flag_clear(peck->eis_behavior, behavior);
+		break;
+	}
+}
+
 void
 peck_enable_ei_behavior(struct peck *peck, enum peck_ei_behavior behavior)
 {
@@ -657,8 +810,6 @@ peck_enable_ei_behavior(struct peck *peck, enum peck_ei_behavior behavior)
 	case PECK_EI_BEHAVIOR_HANDLE_ADDED_SCROLL:
 	case PECK_EI_BEHAVIOR_HANDLE_FRAME:
 	case PECK_EI_BEHAVIOR_HANDLE_SYNC:
-		flag_set(peck->ei_behavior, behavior);
-		break;
 	case PECK_EI_BEHAVIOR_HANDLE_RESUMED:
 	case PECK_EI_BEHAVIOR_HANDLE_PAUSED:
 		flag_set(peck->ei_behavior, behavior);
@@ -666,6 +817,47 @@ peck_enable_ei_behavior(struct peck *peck, enum peck_ei_behavior behavior)
 	}
 }
 
+void
+peck_disable_ei_behavior(struct peck *peck, enum peck_ei_behavior behavior)
+{
+	switch (behavior) {
+	case PECK_EI_BEHAVIOR_NONE:
+		abort();
+		break;
+	case PECK_EI_BEHAVIOR_HANDLE_CONNECT:
+	case PECK_EI_BEHAVIOR_AUTOSEAT:
+		flag_clear(peck->ei_behavior, behavior);
+		break;
+	case PECK_EI_BEHAVIOR_AUTODEVICES:
+		peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_CONNECT);
+		peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTOSEAT);
+		peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_ADDED);
+		peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_RESUMED);
+		break;
+	case PECK_EI_BEHAVIOR_HANDLE_ADDED:
+		peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_ADDED_POINTER);
+		peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_ADDED_POINTER_ABSOLUTE);
+		peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_ADDED_KEYBOARD);
+		peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_ADDED_TOUCH);
+		peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_ADDED_BUTTON);
+		peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_ADDED_SCROLL);
+		break;
+	case PECK_EI_BEHAVIOR_AUTOSTART:
+	case PECK_EI_BEHAVIOR_HANDLE_ADDED_POINTER:
+	case PECK_EI_BEHAVIOR_HANDLE_ADDED_POINTER_ABSOLUTE:
+	case PECK_EI_BEHAVIOR_HANDLE_ADDED_KEYBOARD:
+	case PECK_EI_BEHAVIOR_HANDLE_ADDED_TOUCH:
+	case PECK_EI_BEHAVIOR_HANDLE_ADDED_BUTTON:
+	case PECK_EI_BEHAVIOR_HANDLE_ADDED_SCROLL:
+	case PECK_EI_BEHAVIOR_HANDLE_FRAME:
+	case PECK_EI_BEHAVIOR_HANDLE_SYNC:
+	case PECK_EI_BEHAVIOR_HANDLE_RESUMED:
+	case PECK_EI_BEHAVIOR_HANDLE_PAUSED:
+		flag_clear(peck->ei_behavior, behavior);
+		break;
+	}
+}
+
 static inline void
 peck_create_eis_seat(struct peck *peck, struct eis_client *client)
 {
