@@ -49,6 +49,14 @@ MUNIT_TEST(test_ei_ref_unref)
 	return MUNIT_OK;
 }
 
+MUNIT_TEST(test_ei_unref_immediately)
+{
+	struct peck *peck = peck_new();
+	peck_unref(peck);
+
+	return MUNIT_OK;
+}
+
 MUNIT_TEST(test_ei_disconnect_immediately)
 {
 	_unref_(peck) *peck = peck_new();
@@ -524,7 +532,7 @@ MUNIT_TEST(test_ei_disconnect_after_unbind_after_received)
 
 MUNIT_TEST(test_client_is_sender)
 {
-	_unref_(peck) *peck = peck_new_context(PECK_EI_SENDER);
+	_unref_(peck) *peck = peck_new_context("mode", PECK_EI_SENDER);
 
 	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_NONE);
 	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_HANDLE_SYNC);
@@ -544,7 +552,7 @@ MUNIT_TEST(test_client_is_sender)
 
 MUNIT_TEST(test_client_is_receiver)
 {
-	_unref_(peck) *peck = peck_new_context(PECK_EI_RECEIVER);
+	_unref_(peck) *peck = peck_new_context("mode", PECK_EI_RECEIVER);
 
 	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_NONE);
 	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_HANDLE_SYNC);
@@ -922,6 +930,76 @@ MUNIT_TEST(test_ei_ping)
 	return MUNIT_OK;
 }
 
+MUNIT_TEST(test_ei_ping_delayed_pong)
+{
+	_unref_(peck) *peck = peck_new();
+	uint32_t ping_id_1 = 0, ping_id_2 = 0;
+
+	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ACCEPT_ALL);
+	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_ADD_KEYBOARD);
+	peck_enable_eis_behavior(peck, PECK_EIS_BEHAVIOR_RESUME_DEVICE);
+
+	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTODEVICES);
+	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_AUTOSTART);
+	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_ADDED);
+	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_ADDED_KEYBOARD);
+	peck_enable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_RESUMED);
+
+	peck_dispatch_until_stable(peck);
+
+	peck_disable_eis_behavior(peck, PECK_EIS_BEHAVIOR_HANDLE_SYNC);
+	peck_disable_eis_behavior(peck, PECK_EIS_BEHAVIOR_HANDLE_FRAME);
+	peck_disable_ei_behavior(peck, PECK_EI_BEHAVIOR_HANDLE_SYNC);
+
+	with_client(peck) {
+		peck_drain_ei(ei);
+
+		/* We send two ping events, one before and one after the frame */
+		struct ei_device *device = peck_ei_get_default_keyboard(peck);
+		_unref_(ei_ping) *ping1 = ei_new_ping(ei);
+		_unref_(ei_ping) *ping2 = ei_new_ping(ei);
+		ping_id_1 = ei_ping_get_id(ping1);
+		ping_id_2 = ei_ping_get_id(ping2);
+		ei_ping(ping1);
+		ei_device_keyboard_key(device, 123, true);
+		ei_device_frame(device, ei_now(ei));
+		ei_ping(ping2);
+	}
+
+	peck_dispatch_until_stable(peck);
+
+	with_server(peck) {
+		/* check events arrive in the right order, without processing them */
+		struct eis_event *ping1 = peck_eis_next_event(eis, EIS_EVENT_SYNC);
+		_unref_(eis_event) *key = peck_eis_next_event(eis, EIS_EVENT_KEYBOARD_KEY);
+		_unref_(eis_event) *frame = peck_eis_next_event(eis, EIS_EVENT_FRAME);
+		struct eis_event *ping2 = peck_eis_next_event(eis, EIS_EVENT_SYNC);
+
+		eis_event_unref(ping1);
+
+		/* Send the modifiers event as if it was processed between the two pings. */
+		eis_device_keyboard_send_xkb_modifiers(eis_event_get_device(key), 0x1, 0x2, 0x3, 0x4);
+
+		eis_event_unref(ping2);
+	}
+
+	peck_dispatch_until_stable(peck);
+
+	with_client(peck) {
+		_unref_(ei_event) *pong1 = peck_ei_next_event(ei, EI_EVENT_PONG);
+		_unref_(ei_event) *mods = peck_ei_next_event(ei, EI_EVENT_KEYBOARD_MODIFIERS);
+		_unref_(ei_event) *pong2 = peck_ei_next_event(ei, EI_EVENT_PONG);
+
+		struct ei_ping *ping1 = ei_event_pong_get_ping(pong1);
+		struct ei_ping *ping2 = ei_event_pong_get_ping(pong2);
+
+		munit_assert_int64(ping_id_1, ==, ei_ping_get_id(ping1));
+		munit_assert_int64(ping_id_2, ==, ei_ping_get_id(ping2));
+	}
+
+	return MUNIT_OK;
+}
+
 MUNIT_TEST(test_ei_ping_within_frame)
 {
 	_unref_(peck) *peck = peck_new();
