@@ -215,15 +215,15 @@ class Ei:
         """
 
         def seat_cap(seat, mask, intf_name):
-            seat._interface_masks[intf_name] = mask
+            seat.interface_masks[intf_name] = mask
 
-        seat._interface_masks = {}
+        seat.interface_masks = {}
         seat.connect("Capability", seat_cap)
 
         def bind_mask(interfaces: list[InterfaceName]) -> int:
             return reduce(
                 lambda mask, v: mask | v,
-                [seat._interface_masks[i] for i in interfaces],
+                [seat.interface_masks[i] for i in interfaces],
                 0,
             )
 
@@ -337,7 +337,7 @@ class Eis:
         stdout, stderr = None, None
         for _ in kill_gently(self.process):
             try:
-                stdout, stderr = self.process.communicate(timeout=1)
+                stdout, stderr = self.process.communicate(timeout=3)
                 break
             except subprocess.TimeoutExpired:
                 pass
@@ -571,6 +571,8 @@ class TestEiProtocol:
             InterfaceName.EI_POINTER_ABSOLUTE,
             InterfaceName.EI_KEYBOARD,
             InterfaceName.EI_TOUCHSCREEN,
+            InterfaceName.EI_SCROLL,
+            InterfaceName.EI_BUTTON,
         ]:
             ei.send(
                 setup.InterfaceVersion(interface, VERSION_V(1))
@@ -682,7 +684,21 @@ class TestEiProtocol:
         connection = ei.connection
         assert connection is not None
         seat = ei.seats[0]
-        ei.send(seat.Bind(0x1))  # binding to invalid caps should get us disconnected
+
+        first_invalid_mask = None
+        for i in range(64):
+            mask = 1 << i
+            if mask not in seat.interface_masks.values():
+                first_invalid_mask = mask
+                break
+        else:
+            assert False, (
+                "EIS implementation has bound all 64 bits, cannot find unused bitmask"
+            )
+
+        ei.send(
+            seat.Bind(first_invalid_mask)
+        )  # binding to invalid caps should get us disconnected
         try:
             ei.dispatch()
             time.sleep(0.1)
